use futures::Future;
use nostr_rs_client::{Clients, Error as ClientError};
use nostr_rs_storage::Db;
use nostr_rs_types::{client::Subscribe, Request, Response};
use std::pin::Pin;
use tokio::sync::mpsc;

#[derive(Debug, thiserror::Error)]
pub enum Error {
    #[error("Nostr: {0}")]
    Addr(#[from] nostr_rs_types::types::addr::Error),

    #[error("client: {0}")]
    Client(#[from] ClientError),
}

fn on_connection(
    sent_messages: Vec<Request>,
    socket: mpsc::Sender<Request>,
) -> Pin<Box<dyn Future<Output = ()> + Send>> {
    println!("REconnecting with {:?}", sent_messages);
    Box::pin(async move {
        for m in sent_messages {
            let _ = socket.send(m).await;
        }
    })
}

#[tokio::main]
async fn main() {
    env_logger::init();
    let clients = Clients::default();
    clients.send(Subscribe::default().into()).await;

    let _ = clients
        .connect_to("wss://relay.damus.io/", Some(on_connection))
        .await;
    let db = Db::new("./db").expect("db");

    loop {
        println!("going into loop");
        if let Some((msg, _relayed_by)) = clients.recv().await.expect("valid connection") {
            match msg {
                Response::Event(x) => {
                    let event = x.event;

                    let _ = db.store(&event);
                }
                _ => {}
            }
        }
    }
}
